/*
 * Bob++/include/bob_id.h
 *
 *  These classes are used to store all the identfiers
 */
#ifndef BOBIDENTIF
#define BOBIDENTIF

#include <limits.h>

namespace Bob {

/** 
 *  @defgroup IdGRP The Bob++ identifiers
 *  @ingroup CoreGRP
 *  @brief the different opject identifiers used in Bob++
 *  @{
 *  To be able to print some human readable statistics/loggin information,
 *  Bob++ defines several forms of identifiers.
 *  The EnvId class is used to get a char * that named a computing ressource
 *  as threaded, process...
 *  
 *  The Id is more general and is used to named the different statistics of 
 *  algorithms or data structures operations 
 *  One specific user algorithm has its statistics, but also the genchild, 
 *  and the different data structure as Priority Queue and Goal.
 */


/** The EnvId is an abstract class to print the identifier of 
 * a computing ressource in a environment. 
 *
 * An object of this class must be redefined in eachenvironment and must 
 * be set in the Bobpp log using the core::log().setEid()
 * 
 */
class EnvId {
   public:
   /** Constructor
     */
   EnvId() {}
   /** Destructor
     */
   virtual ~EnvId() {}
   /** method that returns the id of the thread/process according to 
     * the environment.
     */
   virtual long id()=0;
   /** method that returns the number of the thread/process according to 
     * the environment.
     */
   virtual long nb()=0;
};

/** The SeqId stores the default id that is stored in the Log().
 */
class SeqId :public EnvId {
   
   public:
      /** Constructor
        */
      SeqId() :EnvId() {}
      /** Destructor
        */
      virtual ~SeqId() { }
      /** Here the method simply returns a char * buffer with a "-"
        */
      virtual long id() { return 0L; }
      /** Method that returns the number of the thread/process according to 
        * the environment.
        */
      virtual long nb() { return 1L; }
};


/** the Node identifier class
 * The class is used to associate an unique identifier to a node.
 * It is used to be able to reconstruct the search space.
 * This identifier is written in the log file by the base_node::log_space() method.
 */
class NodeId {
   long id;
   public:
      /** constructor
        */
      NodeId() : id(0) { }
      /** constructor
        */
      ~NodeId() { }
      /** get the node identifier as a long
        */
      long &get() { return id; }
      /** create a unique node identifier according  to the environment id, and the value n.
        */
      void create(EnvId &eid, int n) { 
         
      }
};

/** The abstract class for an identifier
 */
class Id {
   protected:
      /// Buffer used to store the name, returns by the str() method.
      strbuff<50> st;
   public:
      /// Constructor
      Id() : st(){ }
      /// Constructor
      Id(const Id &i) : st(i.st){ }
      /// destructor
      virtual ~Id() {}
      /// Method to get the complete string identifier.
      virtual const char *str() const=0;
      /// Method to get the simple string identifier.
      virtual const char *id() const=0;

};
/** The class for a char identifier with a parent identifier
 */
class CId : public Id {
   protected:
      /// Buffer used to store the name, returns by the id() method.
      strbuff<5> n;
      /** an Id pointer to know the Id father of this Id. 
        * This pointer could be equal to 0 if no father is defined.
        */
      Id *father;
   public:
      /// Constructor
      CId():Id(),n(),father(0) { 
         //DBGAFF_MEM("CId::CId()","Constructor");
      }
      /// Constructor
      CId(char const _n,Id *f):Id(),n(_n),father(f) { 
         update();
         //DBGAFF_MEM("CId::CId(char const,Id*)","Constructor");
      }
      /// Constructor
      CId(char const _n):n(_n),father(0) { 
         update();
         //DBGAFF_MEM("CId::CId(char )","Constructor");
      }
      /// Constructor
      CId(const CId &cid):Id(cid),n(cid.n),father(cid.father) { 
         update();
         //DBGAFF_MEM("CId::CId(CId &)","Constructor");
      }
      /// destructor
      virtual ~CId() {
         //DBGAFF_MEM("CId::~CId","Destructor");
      }
      /// get th identifier of the stat (a char)
      const char get() const { return *(n.str()); }
      /// set the name of the Identificator and the father identificateur
      void set(char _n,Id *f) { n.flush();n<<_n; father=f;update();}
      /// set the name of the Identificator 
      void set(char _n) { n.flush();n<<_n;update(); }
      /// set the name of the Identificator
      void set(Id *i) { father=i;update(); }
      /// Update the internal strbuff.
      void update() {
         st.flush();
         if ( father!=0 ) st << father->str()<<":";
         st << n; 
      }
      /// Method to put the CId on a strbuf
      virtual const char *str() const {
         return st.str();
      }
      /// Method to put the Id on a strbuf
      virtual const char *id() const {
         return n.str();
      }

};

/** The class for a string identifier with a parent identifier
 */
class SId : public Id {
   protected:
      /// Buffer used to store the name returns by the id() method.
      strbuff<10> n;
      /** an Id pointer to know the Id father of this Id. 
        * This pointer could be equal to 0 if no father is defined.
        */
      Id *father;
      /** method used to update the internal buffer returns by 
        * the id() or str() methods.
        */
      void update() {
         st.flush();
         if ( father!=0 ) st << father->str()<<":";
         st << n; 
      }

   public:
      /// Constructor
      SId():n(""),father(0) { 
         //DBGAFF_MEM("Sid::Sid()","Constructor");
      }
      /// Constructor
      SId(std::string const &_n):n(_n),father(0) { 
         update();
         //DBGAFF_MEM("Sid::Sid(string n)","Constructor");
      }
      /// Constructor
      SId(std::string const &_n,Id *f):n(_n),father(f) { 
         update();
         //DBGAFF_MEM("Sid::Sid(string n,Id*)","Constructor");
      }
      /// Constructor
      SId(char *const &_n):n(_n),father(0) { 
         update();
         //DBGAFF_MEM("Sid::Sid(char *)","Constructor");
      }
      /// Constructor
      SId(char *const &_n,Id *f):n(_n),father(f) { 
         update();
         //DBGAFF_MEM("Sid::Sid(char *,Id*)","Constructor");
      }
      /// Constructor
      SId(const SId &sid):n(sid.n),father(0) { 
         update();
         //DBGAFF_MEM("Sid::Sid(Sid &)","Constructor");
      }
      /// destructor
      virtual ~SId() {
         //DBGAFF_MEM("Sid::~Sid","Destructor");
      }
      /// set the name of the Identificator and the father identificateur
      void set(std::string &_n,Id *f) { n.flush();n.add(_n.c_str()); father=f;update();}
      /// set the name of the Identificator 
      void set(std::string &_n) { n.flush();n.add(_n.c_str());update(); }
      /// set the name of the Identificator
      void set(char *_n) {  n.flush();n.add(_n);update(); }
      /// set the name of the Identificator and the father identificateur
      void set(char *_n,Id *f) { n.flush();n.add(_n);father=f;update(); }
      /// set the name of the Identificator
      void set(Id *i) { father=i;update(); }
      /// Method to put the CId on a strbuf
      virtual const char *str() const {
         return st.str();
      }
      /// Method to put the Id on a strbuf
      virtual const char * id() const {
         return n.str();
      }
};

/**
  * @}
  */
}; // end of namspace



#endif
