/*
 * Bob++/bob_log.h
 *
 *  These classes are used for the logging information of the bob library
 */
#ifndef BOBLOG_ABSTRACT
#define BOBLOG_ABSTRACT

#include <sstream>
#include <sys/time.h>
#include <time.h>
#include <pthread.h>
#include <iostream>

//#include <thr/thr.h>



namespace Bob {

/** @defgroup LOGGRP The Bob++ Logging classes
 *  @ingroup IOGRP
 *  @{
 *  It has been decided that each process creates one log file.
 *  Then, the information stored in the file could be produced by
 *  only one process, with only one thread but also from several threads... 
 *  We would that log file stores a maximum of information
 *  Mainly, these information are generated using the statistics objects
 *  (see @ref STATGRP).
 *
 *  The different information to log are :
 *   - the activities of threads/processors to solve an instance 
 *     of a problem using an algorithm. The data to store are:
 *       - the beginning and the end time of of an algorithm
 *       - the number of time genchild::operator() has been executed
 *       - the statistics associated with the algorithm
 *       - the number of access on the local or shared data structures
 *   - the number of operation on one shared data structure
 *   
 *
 *   To identify the different components, we associated an identifier to 
 *   each of them.
 *   we dissociate two of them :
 *    - the envid the identifier associated with a thread or a process 
 *    (according to the environment). An envid is hierarchical, meaning that, 
 *    for example, it is composed by a process id and a thread id or is equal 
 *    to only one process id.
 *    - a Oid an object identifier, an oid represents :
 *          - an instance :  to identify the resolution of an instance of a 
 *             problem.
 *          - an algorithm : associated with an instance on different envid
 *               (EnvId:InstanceId:"BB"|"DC"|"CDC"|"DP"|"AS")
 *          - a genchild : associated with an algorithm 
 *               (EnvId:InstanceId:AlgoId:"GC")
 *          - a data structure : associated with an envid. Data structure 
 *            is the more complicated object to identify, because it could be
 *            associated with a thread to collect the activity of the thread
 *            on the data structure or with a process. 
 *            The data structure is then shared beween the threads and then
 *            the statistics represent the sum of the activity of the 
 *            threads that are associated with the process.
 *       
 *
 */

/** default size for the template of the strbuff class
 */
#define MAXBUF 1024

/** The Bob++ strbuff class
  *
  * This class has been implemented to avoid the use of the std::string
  * in the code. The std::string use many memory allocation then in a multithreaded
  * environment, malloc calls yield to poor performances.
  *
  * The size of the internal buffer is givn using the template parameter.
  * The default size is stored in the MAXBUF macro.
  *
  * We have also add conversion method from other basic types.
  */
template<int n=MAXBUF>
class strbuff {
char *e;
char b[n+1];

   void tcapa(int i) { 
      if ( (e-b)>=n ) { 
         std::cerr <<"N:"<<(e-b)<<" a "<<i<<"\n";
         e=b+n;*e=0;
         std::cerr <<"Warning Capacity :("<<(e-b)<<"/"<<n<<":|"<<b<<"|\n"; 
      } 
   }
public: 
   /** Constructor
     */
   strbuff() : e(b) { *b=0;}
   /** Constructor
     * @param s a char * to initialize s.
     */
   strbuff(const char s):e(b) { add(s); }
   /** Constructor
     * @param s a char * to initialize s.
     */
   strbuff(const char *s):e(b) { add(s); }

   /** Constructor
     * @param s an strbuf<n>
     */
   strbuff(const strbuff<n> &s):e(b) { add(s); }
   /** Constructor
     * @param s a char * to initialize s.
     */
   strbuff(const std::string &s):e(b) { add(s.c_str()); }

   /** returns the data in a char * type.
     */
   const char *str() const { return b; }
   /** return the size of the internal data
     */
   int size() const { return e-b; }
   /** add method
     * @param s the string to add to the strbuff
     */
   void add(const  std::string s) { strcpy(e,s.c_str());e+=strlen(s.c_str());tcapa(1); }
   /** add method
     * @param s a char * to add to the strbuff
     */
   void add(const  char *s) { strcpy(e,s);e+=strlen(s);tcapa(2); }
   /** add method
     * @param c a char to add to the strbuff
     */
   void add(const  char c) { *e=c;e++;*e=0;tcapa(3); }
   /** add method
     * @param i the int to add to the strbuff
     */
   void add(const  int i) { sprintf(e,"%d",i);e+=strlen(e);tcapa(4); }
   /** add method
     * @param l the long to add to the strbuff
     */
   void add(const  long l) { sprintf(e,"%ld",l);e+=strlen(e);tcapa(5); }
   /** add method
     * @param l the long long to add to the strbuff
     */
   void add(const long long l) { sprintf(e,"%lld",l);e+=strlen(e);tcapa(10); }
   /** add method
     * @param l the unsigned long long to add to the strbuff
     */
   void add(const unsigned long long l) { sprintf(e,"%llu",l);e+=strlen(e);tcapa(6); }
   /** add method
     * @param f the float to add to the strbuff
     */
   void add(const  float f) { sprintf(e,"%f",f);e+=strlen(e);tcapa(7); }
   /** add method
     * @param d the double to add to the strbuff
     */
   void add(const  double d) { sprintf(e,"%lf",d);e+=strlen(e);tcapa(8); }
   /** add method
     * @param s an strbuff to add to string
     */
   template<int nt>
   void add(strbuff<nt> const &s) { strcpy(e,s.str());e+=s.size();tcapa(9); }

   /** operator<<
     * @param t a templatized variable to add to the strbuff.
     */
   template<class T>
   strbuff<n> & operator<<(const T&t) { add(t); return *this; }

   /** the classical operator to output a strbuff
     * @param os the output stream
     * @param s the strbuff to be displayed
     */
   friend std::ostream &operator<<(std::ostream &os, const strbuff<n> &s) {
      return os << s.b;
   }

   /** flush method
     * method to empty the buffer.
     */
   void flush() { *b=0;e=b; }
};

/** The Log_base class
 * This class is a virtual version and do nothing.
 */
class Log_base {
	public:
      /// constructor
		Log_base() {}
      /// destructor
		virtual ~Log_base() {}
      /** Display a strbuff 
        * @param s the strbuff to display.
        */
      virtual void out(strbuff<> const &s){}
};

/** The Log_file class
 * This class write the information on a file
 */
class Log_file :public Log_base{
	public:
      /// constructor
		Log_file(std::string file):Log_base(),os(file.c_str(),std::ios::trunc) { }
      /// destructor
		virtual ~Log_file() { os.close(); }
      /** Display a strbuff 
        * @param lm the strbuff to display.
        */
      virtual void out(strbuff<> const &lm);
    protected:
      /// the ostream to save all logging information on a file
      std::ofstream os;
};

#ifndef WIN32
/** The Log_net class
 * This class send the log to a network host
 */
class Log_net :public Log_base{
  protected:
       /// Output socket id
     int os;
  public:
     /// constructor
     Log_net(std::string host, int port);
     /// destructor
     virtual ~Log_net();
      /** Display a strbuff 
        * @param lm the strbuff to display.
        */
     virtual void out(strbuff<> const &lm);
};
#endif

class EnvId;
class SeqId;

/** The Log class
 * This class is an interface for the real Log object that could be 
 * a simple Log_base object (it does nothing) or a Log_file.
 */
class Log :public Log_base{
	public:
      /// Constructor
		Log();
      /// Destructor 
		virtual ~Log();
      /** Set the log object
        * @param lb the real log used by the wrapper
        */
      void setLog(Log_base *lb);
      /** Set the log object
        * @param _ei the environment Id of the log
        */
      void setEId(EnvId *_ei) ;
      /** Lock the log for a thread
        */
      void lock() { pthread_mutex_lock(&mut); }
      /** Unlock the log for a thread
        */
      void unlock() { pthread_mutex_unlock(&mut); }
      /** Display a strbuff without lock the log
        */
      virtual void out_nolock(strbuff<> const &lm);
      /** Display a strbuff in an critical section.
        * @param lm the strbuff to display.
        */
      virtual void out(strbuff<> const &lm);
      /** Display a strbuff in an critical section.
        * @param eb set this parameter to true if you want to display the Envid 
        * @param lm the strbuff to display.
        */
      virtual void out(bool eb,strbuff<> const &lm);
      /** Display the id
        * @param s a char * that stores a statistic root identifier.
        * @param c the char that stores a statistic identifier
        * @param n the value of the statistic
        * @param g a boolean indicating if the statistic is global or local to a processor
        */
      virtual void out(const char *s,char c,int n,bool g=false);
    protected:
      /// The real log class
      Log_base *l;
      /// the mutex
      pthread_mutex_t mut;
      /// The EnvId environment identifier
      EnvId *ei;
};

/**
 * @}
 */

};
#endif
