/*
 * Bob++/include/BobStat.h
 *
 *  These classes are used to store de statistics of the algorithms
 */
#ifndef BOBSTAT
#define BOBSTAT

namespace Bob {

/** @defgroup STATGRP The Bob++ statistics classes
 *  @ingroup CoreGRP
 *
 *  This module is a part of the core of the Bob++ library
 *
 *  This module proposes a set of classes to store the statistics
 *  about the different activities.
 *
 *  We have choosen to store time statistics (@ref stat_time)
 *  We have choosen to store counter (@ref stat_counter)
 *  The class stat could store several stat_time, stat_counter
 *  and also pointers on other stat objects.
 *
 *  A stat object is associate with each bob++ components.
 *  For example, a BBAlgo object use a stat object which is a BBStat.
 *  A BBStat object inherits indirectly from the stat object. In
 *  the constructor, some  stat_counter or stat_time objects are added using
 *  respectively add_counter() or add_time() stat method.
 *  @{
 */

/// Buffer used to store the complete Id of a stat.
typedef strbuff<30> name_st;
/// Buffer used to display all the labels of the stats on te header log.
typedef strbuff<> buff_st;
/**
 * The stat_time class stores the time execution of a function
 * or something. It stores an identifier the methods start() and end() may
 * be used to indicate the beginning and the end of the component we want
 * to profile.
 * These methods use the core::dtime() method to get the time.
 *
 */
class stat_time : public CId {
protected:
  /// The number of runs
  int nbr;
  /// Start time
  double st;
  /// current time
  double ct;
  /// delta time
  double dt;
  /// cumulating timers
  double cumt;
  /// long name of the stat
  name_st nst;
  /// true if the stat is global
  bool global;
public:
  /// Constructor
  stat_time(): CId(), nbr(0), st(0), ct(0), dt(0), cumt(0), nst("def"), global(false) {
    DBGAFF_MEM("stat_time::stat_time", "Constructor()");
  }
  /// Constructor
  stat_time(const stat_time &stt): CId(stt), nbr(stt.nbr), st(stt.st), ct(stt.ct), dt(stt.dt), cumt(stt.cumt), nst(stt.nst), global(false) {
    DBGAFF_MEM("stat_time::stat_time", "Const stat_time");
  }
  /// Constructor
  stat_time(char s): CId(s), nbr(0), st(0), ct(0), dt(0), cumt(0), nst("def"), global(false) {
    DBGAFF_MEM("stat_time::stat_time", "Constructor(char *)");
  }
  /// Constructor
  stat_time(char s, const char *_nst): CId(s), nbr(0), st(0), ct(0), dt(0), cumt(0), nst(_nst), global(false) {
    DBGAFF_MEM("stat_time::stat_time", "Constructor(char *)");
  }
  /// Destructor
  virtual ~stat_time() {
    DBGAFF_MEM("stat_time::~stat_time", "Destructor");
  }
  /// Intialize the Statistics
  void init(char _n, Id *s, bool _global) {
    set(_n, s);
    nbr = 0;
    st = ct = dt = cumt = 0.0;
    global = _global;
  }
  /// Intialize the Statistics
  void init(char _n, Id *s, const char *_nst, bool _global) {
    set(_n, s);
    nbr = 0;
    st = ct = dt = cumt = 0.0;
    global = _global;
    nst.flush();nst.add(_nst);
  }
  /// Set the start time
  void start();
  /// Set the end time
  void end();
  /** merge method use to collect the statistics from another stat.
    * @param st the stat_counter to collect
    */
  void merge(const stat_time &st) {
    nbr += st.nbr;
    cumt += st.cumt;
  }
  /// gets the cumulating time
  double getc() const {
    return cumt;
  }
  /// gets the number of utilization of this timers
  double getn() const {
    return nbr;
  }
  /** Pack method to Serialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void Pack(Serialize &bs) const {
    DBGAFF_ENV("Stat_time::Pack", "Packing time stat");
    bs.Pack(&nbr, 1);
    bs.Pack(&st, 1);
    bs.Pack(&ct, 1);
    bs.Pack(&dt, 1);
    bs.Pack(&cumt, 1);
  }
  /** Unpack method to deserialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void UnPack(DeSerialize &bs)  {
    DBGAFF_ENV("Stat_time::UnPack", "Unpacking time stat");
    bs.UnPack(&nbr, 1);
    bs.UnPack(&st, 1);
    bs.UnPack(&ct, 1);
    bs.UnPack(&dt, 1);
    bs.UnPack(&cumt, 1);
  }
 
  /** Put the id, the name, and the type of the stat on a buffer
    * @param os the buffer
    */
  void name(buff_st &os) const { 
     os<<(global?"g:":"l:")<<str()<<":TIME:"<<nst<<"\n"; 
  }

  /** Display the stat_time in an ostream
    * @param o the ouput
    * @param s the stat_time to display
    */
  friend std::ostream& operator<< (std::ostream& o, const stat_time& s);
};

/**
 * The stat_counter class could store a counter
 */
class stat_counter : public CId {
protected:
  /// the counter
  unsigned long long nb;
  /// the maximum
  unsigned long long max;
  /// the minimum
  unsigned long long  min;
  /// long name of the stat
  name_st nst;
  /// Boolean to tell if the statistics is local to a envirement or global
  bool global;
public:
  /// Constructor
  stat_counter() : CId(), nb(0L), max(0L), min(0L),nst("def"),global(false) {}
  /// Constructor
  stat_counter(const stat_counter &sc) : CId(sc), nb(sc.nb), max(sc.max), min(sc.min),nst(sc.nst),global(sc.global) {}
  /// Destructor
  virtual ~stat_counter() {}
  /** Initialize the stat_counter
    * @param _n a char * to identify the counter
    * @param i the parent identifier
    */
  void init(char _n, Id *i) {
    set(_n, i);
    nb = max = min = 0;
  }

  /** Initialize the stat_counter
    * @param _n a char * to identify the counter
    * @param i the parent identifier
    * @param _nst the label of the Statistic.
    * @param glob boolean tells if the stats is local to the process.
    */
  void init(char _n, Id *i,const char *_nst,bool glob=false) {
    set(_n, i);
    nb = max = min = 0;
    nst.flush();nst.add(_nst);
    global=glob;
  }
  /** add a value to the counter
    * @param v the value to add
    */
  void add(int v = 1) {
    nb += v;
    update_log();
  }
  /** ++ operator
    */
  void operator++(int n) {
    nb++;
    update_log();
  }
  /** -- operator
    */
  void operator--(int n) {
    nb--;
    update_log();
  }
  /** += operator
    * @param n the value to add
    */
  void operator+=(int n) {
    nb += n;
    update_log();
  }
  /** += operator
    * @param n the value to remove
    */
  void operator-=(int n) {
    nb -= n;
    update_log();
  }
  /** Method to update internal value
    */
  void update_mix();
  /** Method to update internal value and display the value to the log
    */
  void update_log();
  /** get the value
    */
  unsigned long long  get() const {
    return nb;
  }
  /** get the maximal of the value
    */
  unsigned long long get_max()const  {
    return max;
  }
  /** merge method use to collect the statistics from another stat.
    * @param sc the stat_counter to collect
    */
  void merge(const stat_counter &sc) {
    nb += sc.nb;
    update_mix();
  }
  /** Pack method to Serialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void Pack(Serialize &bs) const {
    DBGAFF_ENV("stat_counter::Pack", "Packing stat counter");
    bs.Pack(&nb, 1);
    bs.Pack(&max, 1);
    bs.Pack(&min, 1);
  }
  /** Unpack method to deserialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void UnPack(DeSerialize &bs)  {
    DBGAFF_ENV("stat_counter::UnPack", "Unpacking stat counter");
    bs.UnPack(&nb, 1);
    bs.UnPack(&max, 1);
    bs.UnPack(&min, 1);
  }

  /** Put the id, the name, and the type of the stat on a buffer
    * @param os the buffer
    */
  void name(buff_st &os) const { 
      os<<(global?"g:":"l:")<<str()<<":COUNT:"<<nst<<"\n"; }

  /// operator to display the statistics
  friend std::ostream& operator<< (std::ostream& o, const stat_counter& s);
};


class stat;

/** MapCount is the type which associate a stat_counter with a
  * char key in a std::map.
  */
typedef std::map<char, stat_counter> MapCount;
/** MapTime is the type which associate a stat_time with a
  * char key in a std::map.
  */
typedef std::map<char, stat_time> MapTime;
/** MapStat is the type which associate a stat * with a
  * char key in a std::map.
  */
typedef std::map<char, stat *> MapStat;

/** The main statistics class
 *
 * The class is the main statiitics class. It could store
 * several stat_time, stat_counter and other stat objects.
 * A counter could be added with the add_counter method
 * A timer could be added with the add_time method
 */
class stat : public CId {
protected:
/// the map that stores all the counters
  MapCount mc;
/// the map that stores all the timers
  MapTime mt;
/// the map that stores all the others stats object associated with this one
  MapStat mss;
  /// long name of the stat
  name_st nst;

public:
  /// Constructor
  stat(): CId(), mc(), mt(), mss(),nst("def") {
    DBGAFF_MEM("stat::stat()", "Constructor");
  }
  /// Constructor
  stat(const stat &st): CId(st), mc(st.mc), mt(st.mt), mss(st.mss),nst(st.nst) {
    DBGAFF_MEM("stat::stat(cons stat)", "Constructor");
  }
  /** Constructor
    * @param _n the identifier for this statitic
    */
  stat(char _n): CId(_n), mc(), mt(), mss(),nst("def") {
    DBGAFF_MEM("stat::stat(char *)", "Constructor");
  }
  /** Constructor
    * @param _n the identifier for this statitic
    * @param f the father identifier
    */
  stat(char _n, Id *f): CId(_n, f), mc(), mt(), mss(),nst("def") {
    DBGAFF_MEM("stat::stat(char *,Id *)", "Constructor");
  }
  /** Constructor
    * @param _n the identifier for this statitic
    * @param _nst the label of the counter.
    */
  stat(char _n,const char *_nst): CId(_n), mc(), mt(), mss(),nst(_nst) {
    DBGAFF_MEM("stat::stat(char *)", "Constructor");
  }
  /** Constructor
    * @param _n the identifier for this statitic
    * @param f the father identifier
    * @param _nst the label of the counter.
    */
  stat(char _n, Id *f,const char *_nst): CId(_n, f), mc(), mt(), mss(),nst(_nst) {
    DBGAFF_MEM("stat::stat(char *,Id *)", "Constructor");
  }
  /// Destructor
  virtual ~stat() {
    mc.clear();
    mt.clear();
    mss.clear();
    DBGAFF_MEM("stat::~stat()", "Destructor");
  }
  /** Initialize the father Id
    * @param i the father identifier
    */
  void init(Id *i) {
    set(i);
    update();
  }
  /** Initialize the father Id
    * @param i the father identifier
    * @param _nst the label of the counter.
    */
  void init(Id *i,char *_nst) {
    set(i);
    nst.flush();nst.add(_nst);
  }
  /** update the internal name of the Id of all stat
   */
  void update();
  /// reset all the statistics
  /// @todo
  void reset() { }
  /** merge method use to collect the statistics from another stat.
    * @param s the stat_counter to collect
    */
  void merge(const stat &s) ;
  /** @name Counter
    *  these methods are for counters.
    */
  //@{
  /** This method adds a counter
    * @param n the name of the counter
    * @param nst the label of the counter.
    * @param global boolean tells if the stats is local to the process.
    */
  void add_counter(char n,const char *nst,bool global=false) {
    mc[n].init(n, this,nst,global);
  }
  /** add a value to the counter identified by n
    * @param n the identifier of the counter to modify
    * @param i the value to add to the counter
    */
  void add(char n, int i) {
    mc[n].add(i);
  }
  /** obtain the counter identifed by the char n
    * @param n the identifier
    */
  stat_counter &get_counter(char n) {
    return mc[n];
  }
  //@}
  /** @name Timer
    *  these methods are for timers.
    */
  //@{
  /** This method adds a timer
    * @param n the name of the timer
    * @param nst the label of the counter.
    * @param global true if this is a global stat
    */
  void add_time(char n,const char *nst, bool global=false) {
    mt[n].init(n, this,nst,global);
  }
  /** obtain the timer identified by the char n
    * @param n the identifier
    */
  stat_time &get_time(char n) {
    return mt[n];
  }
  //@}
  /** @name sub-stats
    *  these methods are for sub-stats.
    */
  /** This method adds a stat
    * @param s the stat object to add
    */
  void add_stat(stat *s) {
    s->init(this);
    mss[s->get()] = s;
  }
  /** obtain the sub-stat identified by the char n
    * @param n the identifier
    */
  stat *get_stat(char n) {
    return mss[n];
  }
  //@}
  /** Pack method to Serialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void Pack(Serialize &bs) const ;
  /** Unpack method to deserialize the SchedNode
    * @param bs the DeSerialize object
    */
  virtual void UnPack(DeSerialize &bs);

  /** @name the Display methods
    */
  //@{
  /// put the id and the name of all the stat stored on the stat object
  void name(buff_st &os)const ;
  /// execute the name methods and put it on the log.
  void namelog() const;

  /** The friend operator<< to display a stat
    */
  friend std::ostream& operator<< (std::ostream& o, const stat& s);
  /** old method to display a stat
    */
  void Prt(ostream &os) {
    os << *this;
  }
  /** old method to display a stat
    */
  void PrtL(ostream &os);
  /** old method to display a stat
    */
  void PrtD(ostream &os);
  //@}
};

/** inline method to display a timer on a std::ostream
 */
inline std::ostream& operator<< (std::ostream& o, const stat_time& s) {
  o << s.str() << "-t:" << s.cumt << "(" << s.nbr << "," << (s.cumt / s.nbr) << ")";
  return o;
}

/** inline method to display a counter on a std::ostream
 */
inline std::ostream& operator<< (std::ostream& o, const stat_counter & s) {
  o << s.str() << "-c:" << s.nb << "(" << s.max << "," << s.min << ")";
  return o;
}

/** inline method to display a stat on a std::ostream
 */
inline std::ostream& operator<< (std::ostream& o, const stat& s) {
  o << s.str() << "-s {\n";
  for (MapCount::const_iterator i = s.mc.begin(); i != s.mc.end() ; i++) {
    o << i->second << "\n";
  }
  for (MapTime::const_iterator i = s.mt.begin(); i != s.mt.end() ; i++) {
    o << i->second << "\n";
  }

  for (MapStat::const_iterator i = s.mss.begin(); i != s.mss.end() ; i++) {
    if (i->second != 0) {
      o << *(i->second) << "\n";
    } else
      o << "NULL\n";
  }
  o << "}\n";
  return o;
}
/**
 * @}
 */

};
#endif
