
/*
 * BOB++
 */

using namespace std;
#include<iostream>
#include<fstream>
#include<cstdlib>
#include<cstring>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/** \mainpage Bob++
    @author Bertrand Le Cun and Fran&ccedil;ois Galea
    @version $(VERSION)

\section MainIntro Introduction

The Bob++ Library is a set of C++ classes. Its goal is to allow easy
implementation of sequential and parallel search algorithms
(Branch and X, Dynamic programming, etc) to solve your own problems.

The official web page of the project is https://bobpp.prism.uvsq.fr

The lastest version of this documentation could be reached at http://bobpp.prism.uvsq.fr

\section MainFeature Features

- Object Oriented
- Several Search Algorithms \ref UserAlgoGRP
- Automatic Options and Parameters Handling ( see @ref OptGRP Module).
- Error Handling
- transparent parallelizations (shared memory, or distributed memory machines)
   (see @ref EnvProgGRP)


\section LibArchi Library Architecture

The Bob++ library is composed of three main parts.
-# The \em core (@ref CoreGRP), includes various features 
   - the options (@ref OptGRP), 
   - the core static class : Bob::core 
   - the data structures that stores objects like nodes and solutions (@ref GDS_GRP and more precisely @ref PQGRP) .
   - 
-# The hierarchy of the different possible algorithms (@ref UserAlgoGRP).
-# The programming environment (@ref EnvProgGRP), which handles the execution
of the algorithm on different distributed, multithreaded or sequential
environments.


\image html archi_bobpp.png
\image latex archi_bobpp.eps "Overview of Bob++ architecture" width=10cm

The Bob++ disributions comes with different examples the description of these
example are given in the \ref Examples page.


\section LibComp Conception of User Algorithms

The idea is to propose a set of classes for each method (search algorithm).
All the proposed algorithms are based on the same scheme :
 - \em Nodes are items in a <em>space search</em>,
 - One of the nodes is known : the \em root node. 
 - A \em method exists to generate nodes from another.

This very simple scheme includes algorithms such as Divide and Conquer,
Branch and X, A*, Dynamic programming, Tabu search, etc...

All of them differ in :
- The \em structure of the space search (tree, graph),
- The <em>scheduling strategy</em> to explore the space search,
- various information available to control the search (evaluation, costs, configuration).

The Bob++ library is based on these basic ideas. The programming of a user 
algorithm can be performed by specializing one of the proposed algorithms.


\section Howto HowTo

To use the library first see the \ref InstMan "Installation page". This
page indicates the procedure to produce the different parts of the library.

*/

/** \page InstMan Getting started with Bob++

This section describes what you can install on the different supported systems,
and how.

- \subpage InstManSupported "Supported platforms"
- \subpage InstManDownload "Downloading the source files"
- \subpage InstManUnx "Configuration, compilation and installation on Un*x systems"
- \subpage InstManConfigure "Configuration options"
- \subpage InstManLocal "Installing Bob++ in a user directory"
- \subpage InstManDocu "Generating the documentation"
- \subpage InstManWin "Compilation and installation on Microsoft Windows systems"



\page InstManSupported Supported Platforms

\section supported_unix UN*X systems

Bob++ runs on many Un*x compatible systems. You may want to use a desktop PC
running a GNU system, a worstation with a proprietary system, a cluster of
workstations as well as computation grids.

Installation can be done through the now classic "configure / make / make
install" process.

\section supported_win Microsoft Windows systems

Bob++ can also be used on Miscrosoft Windows systems, on a limited extent : no
support for clusters of Windows stations is available. Only the sequential and
multithreaded parallel environments can be used.

There is limited support for execution logging on Windows systems : only log
files can be generated. The \ref Logtools "log viewer and log listener" cannot
currently be compiled on Windows systems.

Currently, only project files for the
<a href="http://www.bloodshed.net/devcpp.html">Dev-C++</a> IDE are provided.


\page InstManDownload Downloading the sources files

This section describes the different possible methods to get the Bob++ source
files.

\section downtarball Fetching a tarball

This is the simplest and recommended way to get the source files. You can
download a tarball of the different official Bob++ releases  at the following
URL: http://software.prism.uvsq.fr/releases/bobpp/.

The <a href="http://software.prism.uvsq.fr/releases/bobpp/bobpp-svn.tar.gz">latest
Subversion revision tarball</a> can also be downloaded from there.

We assume that the tarball you have obtained is <tt>bobpp-x.x.x.tar.gz</tt>.
You can extract all the source code with the following command:
\code
      $ gunzip -c bobpp-x.x.x.tar.gz | tar xf -
\endcode

This extracts a <tt>bobpp-x.x.x</tt> directory containing the sources files and all the
necessary configure and make scripts.

\section downsvn Using Subversion and Autotools

You can download the Bob++ source code using
<a href="http://subversion.tigris.org/">subversion</a>, through an anonymous
repository access. This method is not necessary for people who just want to
compile and test Bob++ and some applications. The use of a packaged tarball is
generally sufficient.

Bob++ is based on the GNU
<a href="http://www.gnu.org/software/autoconf/">autoconf</a>, 
<a href="http://www.gnu.org/software/automake/">automake</a> and
<a href="http://www.gnu.org/software/libtool/">libtool</a> tools to simplify the
configuration of the libraries. These tools enable the software designers to
publish source file packages with a \p configure script and a set of \p
Makefile.in files, which handle all the necessary configuration before compiling
the sources and installing the compiled binary files. If you get the source
files from the subversion repository, you need to generate those \p
configure and \p Makefile.in files manually. Thus, your system must have all
these tools installed.

The command for fetching the source code is the following:

\code
      $ svn checkout https://software.prism.uvsq.fr/bobpp/svn/trunk bobpp
\endcode

This generates a \p bobpp directory containing the latest revision of Bob++.

Later on, you can update the source code to the latest revision by issuing the
following shell command:

\code
      $ svn update
\endcode

You can then generate all the necessary configure and make files can be done by
running this command :

\code
      $ ./bootstrap
\endcode

Obviously, your system needs the autoconf, automake and libtool tools to be
installed. If not, better use a tarball of the sources instead.

\page InstManConfigure Configuration options

On Un*x systems, the configuration of the Bob++ library has to be done by
providing arguments to the supplied (or autotools-generated) \c configure
script.

The currently possible options are:

<ul>
<li><tt>--prefix=\<installation dir\></tt> - select the directory where to install
(put) header files, archive, scripts, etc... required to compile and run Bob++
programs.

If not set, Bob++ will be installed globally on your system, in the
standard <tt>/usr/local</tt> directory. Note that you will need root privileges
for such a global installation.

If set, this allows you to install the Bob++ into a user installation
directory, without the necessity for system privileges. More information about
installation in user directories can be found in \ref InstManLocal this page.
</li>
<li><tt>--with-kaapi=\<arg\></tt> - specify whether the kaapi/athapascan
programming environment needs to be generated or not. The provided argument may
one of the following:
<ul>
<li>\c check (default) - generate the Kaapi/Athapascan
module only if Kaapi is installed</li>
<li>\c no - don't generate the kaapi/athapascan module</li>
<li>\c yes - the Kaapi/Athapascan module must be generated. If Kaapi has not
been found, the configure process stops with an error message</li>
<li><tt>path/to/kaapi</tt> - generate the Kaapi/Athapascan module, assuming
the Kaapi root installation directory is <tt>path/to/kaapi</tt>.
</ul></li>

<li> \p --with-glop=\<solver\> - specify which linear solver
has to be used with the Glop library in Bob++. At this time, \c glpk, \c clp, \c
lp_solve, \c cplex or \c xpress are the possible options. Of course the
corresponding Glop module must be installed, otherwise the installation process
will fail.

The default behaviour is to check for the presence of any of the possible glop
modules, and select one of them if found. If the selected module is not the one
you want, use the \p --with-glop option to manually choose the module you want.
</li>
<li> <tt>--with-giw=\<yes|no|check\></tt> specifies whether the <a
href="http://giw.sourceforge.net/">GIW</a> library is present on the system or
not. By default, \p configure checks autoamtically for the presence of GIW.

GIW is only necessary for compiling the \ref bobview "bobview" log
viewing tool. If GIW is not available, bobview will neither be compiled nor
installed.</li>
<li> \p --enable-debug : with this option, Bob++ will display many information during the execution. 
You may deactivate them using command line options of your Bob++ application.
(see @ref DEBUG "here" for more information)
</ul>

Other non-Bob++ specific options are also available. To get a full list of
available options, issue a <tt>./configure --help</tt> command.


\page InstManUnx Configuration, Compilation and Installation on Un*x systems

The whole setup process (configuration, build and installation) follow a now
standard "configure / make / make install" process.

Bob++ relies on an external command called <a
href="http://pkg-config.freedesktop.org/wiki/"><tt>pkg-config</tt></a>. If this
program is not installed on your system, you will not be able to install Bob++.
<tt>pkg-config</tt> is used when compiling Bob++ applications, and is also used
by some of the libraries (Glop, Kaapi, GIW) that some parts of Bob++ depend on.

\section _calling Calling the configure script

Configuring Bob++ consists in executing a \p configure script from the Bob++
source directory, with specific options, like in the example below:

\code
      $ ./configure --prefix=/usr/local/bobpp
\endcode

The above command configures the library of the current host such that the
installation directory would be <tt>/usr/local/bobpp</tt> (which requires root
permission during the last install step).

\see \ref InstManConfigure "List of configuration options"
\see \ref InstManLocal "Installing Bob++ into a user directory"

The result of this command should be a quite long listing of the automatic configuration
of differents tools, libraries, functions, ... and the determination of the C++ compiler.
The configure script uses the default compiler which can be redefined by defining the environment
variable CXX before running configure. For instance, if you want to specify an other C++ compiler,
you can defined CXX environment variable.


\section _make Build the binary files

Compilation of the binaries can be done through this command:
\code
      $ make
\endcode

\section _install Install the binaries and headers

If the previous steps successed, then the installation of the library is done
with the command.
\code
      $ make install
\endcode
Keep in mind you may need privileged permission for installation in a system
dicrectory such as <tt>/usr/local</tt>. Ensure you invoke the command above
having the correct permission enabled, for instance by switching users
beforehand.


\page InstManLocal Installing Bob++ into a user directory

On Un*x systems, the default behaviour for installation is to use a standard
system directory (such as <tt>/usr/local</tt>) as the installation directory. 
This may be useful for a system-wide installation, but it requires the user to
have root access to the system.

An alternative way consists in installing Bob++ into a local user directory.
This is made possible using the \c --prefix option of the \c configure script.
Nowadays, most open source software programs work in the same way, which is
convenient for installing different programs in a common location. However,
additional setup of the user's account is necessary.

This section describes the necessary setup of environment variables in order to
get Bob++ working from a user directory. The variables are standard, which means
this setup will also work for various <tt>autoconf</tt>-based programs and
libraries.

\section InstManLocal_env Environment variables

The list of the different environment variables to be configured is the
following:

- \c PATH - this variable contains the list of all directories where executable
files may be found. It contains different fields separated by colon characters.
You may need to concatenate the defaut value of this variable, which generally
contains the list of system directories for executable commands and programs.

- \c C_INCLUDE_PATH - this variable contains a list of \e additional directories
where C header files may be found. Its default value is empty, which means
header files can only be found in system directories.

- \c CPLUS_INCLUDE_PATH - this varaible is similar to the \c C_INCLUDE_PATH
variable, excepted it concerns C++ header files. You generally may want to set
it up with the same value as \c C_INCLUDE_PATH.

- \c LD_LIBRARY_PATH - this variable contains a list of \e additional
directories where static and dynamic libraries may be found at \e compile time.
It is necessary when compiling programs or libraries which depend on other
libraries (for instance, some setups of Bob++ may depend on Kaapi or MPI
libraries.

- \c LD_RUN_PATH - this variable contains a list of \e additional directories
where dynamic libraries may be found at \e run time. Indeed, dynamic libraries
are not included in the generated binary files, unlike static libraries. Thus,
the system needs to find all necessary dynamic libraries before being able to
run a specific program.

- \c PKG_CONFIG_PATH - this variable contains a list of \e additional
directories where data files for \c pkg-config may be found. \c pkg-config is
widely used by libraries to simplify the compilation of programs which may
depend on various libraries, and is more and more used as standard.

\section InstManLocal_sh Configuring a sh-based user account

We describe here a straightforward method for configuring user accounts which
are based on \c sh compatible shells (\c sh, \c ksh, \c bash ...).

Suppose you have choosen a local directory where all header, library, binary or
documentation files will be installed. Any sub-directory in your home directory
should fit. In the following example we assume that the chosen directory is
<tt>$HOME/local</tt>.

You need to determine which configuration script is executed at login, or when
running a new instance of the shell (when opening a new terminal window for
instance). Standard setups of \c sh call the \c .profile script at login, while
\c ksh and \c bash run a \c .kshrc or \c .bashrc, even when not in a login
shell. For more information, refer to the documentation of the shell you are
using.

To configure all the necessary variables such that <tt>$HOME/local</tt> is used
as an installation directory, add the following lines to your configuration
script:

\code
for i in $HOME/local
do
  test -d $i || continue
  PATH=$i/bin:$PATH
  LD_LIBRARY_PATH=$i/lib:$LD_LIBRARY_PATH
  LD_RUN_PATH=$i/lib:$LD_RUN_PATH
  PKG_CONFIG_PATH=$i/lib/pkgconfig:$PKG_CONFIG_PATH
  C_INCLUDE_PATH=$i/include:$C_INCLUDE_PATH
done

# remove trailing colons in the different variables
export LD_LIBRARY_PATH=${LD_LIBRARY_PATH%:}
export LD_RUN_PATH=${LD_RUN_PATH%:}
export C_INCLUDE_PATH=${C_INCLUDE_PATH%:}
export CPLUS_INCLUDE_PATH=${C_INCLUDE_PATH}
export PKG_CONFIG_PATH=${PKG_CONFIG_PATH%:}
\endcode

Note that multiple installation directories can easily been set up. For this,
just add the different paths to the list in the \c for statement.


\page InstManDocu Generating the documentation

Bob++ uses
<a href="http://www.doxygen.org/">Doxygen</a> to generate its
documentation from the source code files.

On Un*x systems, you can generate the documentation with the command:
\code
      $ make doxygen-doc
\endcode

This line generates the html documentation and the pdf documentation in the
doxygen-doc directory.

The latest version of the documentation can also be browsed online through
<a href="http://bobpp.prism.uvsq.fr/">this link</a>.


\page InstManWin Compilation and installation on Microsoft Windows systems

You can install a subset of the Bob++ modules on top of a Windows based
computer. This will allow you to create sequential and multithreaded Bob++
applications. No support for distributed programming environments is provided.

Windows support is only available for the
<a href="http://www.bloodshed.net/devcpp.html">Dev-C++</a> IDE.

\section _winbuild Build Bob++ library file

Bob++ requires the
<a href="http://devpaks.org/details.php?devpak=194">pthreads-w32 DevPak</a> to
be installed in Dev-C++.

Bob++ is supplied with a \c bobpp.dev project file for Dev-C++. Just open
this project file, click the "build all" button, and the library file will be
generated.

\section _wininstall Install Bob++ in Dev-C++

After having generated the Bob++ library file, you can install it in an easy way
into Dev-C++, through the following steps:

<ol>
<li>Execute the \c build_devpack.bat batch file. This will generate a
<tt>libbobpp-$(VERSION)</tt> directory with \c lib and \c include
subdirectories, among others.</li>
<li>Create a tar archive file containing this directory, with the same name,
plus the \c .tar extension. On Windows, tar archives can be generated using the
<a href="http://www.7-zip.org/">7-zip</a> archiving program.</li> 
<li>Compress this tar archive using the bzip2 method (7-zip also supports bzip2
compression). This will result in a \c libbobpp-$(VERSION).tar.bz2 compressed
archive file.</li>
<li>Rename \c libbobpp-$(VERSION).tar.bz2 into \c libbobpp-$(VERSION).DevPak.
</li>
<li>Install the libbobpp-$(VERSION).DevPak file in Dev-C++.</li>.
</ol>

*/



/** \page Examples Examples
 *
 * <ul>
 * <li> Branch and Bound
 * <ul>
 * <li> \subpage BBSimul : the goal is to generate nodes with a random evaluation from a given value to a maximum value. An invariant is that the evaluation of one node is greater thant the evaluation of its parent node.</li>
 * <li> \subpage MIP : The goal is to solve a Mixed integer program, using Branch and Bound</li>
 * </ul>
 * </li>
 * <li> Divide and Conquer
 * <ul>
 * <li>\subpage NQueens : solve the n-queens problem using a simple divide and conquer</li>
 * <li>\subpage Knapsack : solve a knapsack problem using a simple divide and conquer</li>
 * </ul>
 * </li>
 * </ul>
 *
 */

/** \page BBSimul Simulation of Branch and Bound
 * Here is the simple sequential version of the code. In this code,
 * the same problem is solve two times, to test the two Bob::EnvProg methods.
 * \include examples/SimBB/seq.cpp
 * But there also exists the Multithreaded version
 * \include examples/SimBB/thr.cpp
 * and the Athapascan version
 * \include examples/SimBB/atha.cpp
 */

/** \page MIP Mixed integer programming
 * You could find in the example/TestBB directory the mip.cpp file.
 * This example use the Bob++ Branch and Bound algorithm to solve integer
 * linear program. It uses to evaluate each node a linear solver using 
 * the <a href="http://glop.prism.uvsq.fr/">Glop library</a>. At this time 
 * Glop is not distributed with Bob++.
 *
 * Another directory example/Mip also stores such an example. But in this one we
 * have added the cuts generation. We have implemented the 
 * <a href="http://glock.prism.uvsq.fr/">Glock library</a> to generate cuts
 * on top of the Glop library.
 *
 * These two examples can solve a linear program in lp format
 * \include Mip_Cuts/roy.lp
 *
 */

/** \page NQueens N-Queens problem
 * \include queen/queen.cpp
 */

/** \page Knapsack Knapsack problem
 * \include ks/ks.cpp
 */


/* TutHowToApp Introduction Tutorial on Howto write an application

  This tutorial tries to drive you to write your own application. 
  We first explain all things that are algorithm independant. 
   We do not want here to focus on the Branch and Bound or Price, all the information 
  here are for all Bob++ algorithms But we would considere that the algo you choose has
  the generic name of GLA.
  For a specific algorithms go to @ref UserAlgoGRP.

  section TutHowToStart Getting Started

  subsection TutDir The miscellaneous files and directory

  subsection TutSource The source files

*/

namespace Bob {

/** @defgroup CoreGRP The core of the library
 *  @brief This module stores various groups of class representing 
 *         the core of the library.
 *  @{
 *
 *  This module stores number of sub-modules of the core of the libray.
 */

};
#include<bobpp/bob_conf.h>
#include<bobpp/bob_error.h>
#include<bobpp/bob_io.h>
#include<bobpp/bob_log.h>
#include<bobpp/bob_opt.h>
#include<bobpp/bob_core.h>
#include<bobpp/bob_id.h>
#include<bobpp/bob_stat.h>
/**
 * @}
 */

#include<bobpp/bob_sens.h>
#include<bobpp/bob_balgo.h>
#include<bobpp/bob_gdsbase.h>
#include<bobpp/bob_dspq.h>
#include<bobpp/bob_schedalgo.h>
#include<bobpp/bob_eval.h>
#include<bobpp/bob_cost.h>
#include<bobpp/bob_pri.h>
#include<bobpp/bob_envprog.h>
#include<bobpp/bob_bbalgo.h>
#include<bobpp/bob_dcalgo.h>
#include<bobpp/bob_cdcalgo.h>
#include<bobpp/bob_sdcalgo.h>


#include<bobpp/bob_prob.h>

