
#ifndef BOB_THRENVPROG
#define BOB_THRENVPROG

#include<pthread.h>
#ifndef PTHREAD_STACK_MIN
#define PTHREAD_STACK_MIN  16384
#endif

#include<bobpp/bobpp>

namespace Bob {

/**
 *  @defgroup ThrEnvProgGRP The Multi-threaded programming environment
 *  @brief The multi threaded environment for the Bob++ algorithms
 *  @ingroup EnvProgGRP
 *  @{
 *  \page TEPpage The Multi-threaded programming Environment
 *
 * \section Using
 *
 * The Bob::ThrEnvProg is the static class to spawn the threads.
 * The number of threaded spawned are given in the command line
 * using the --thr -n option.
 *
 * the following code shows how to solve an instance of a problem
 * usng the Divide and Conquer Method.
 * \code
    int main(int n, char ** v) {
        // Define the Threaded environment wrapper
        Bob::ThrBBAlgoEnvProg<MyTrait> env;
        // Initialize the threaded environment
        Bob::ThrEnvProg::init();
        // The initialization of other part of Bob++
        Bob::core::Config(n,v);
        // Start the environment
        Bob::ThrEnvProg::start();

        // Initialize the instance.
        MyInstance *Instance=new MyInstance();

        // Solve the instance i.e. all the tree
        env(Instance);

        // Terminate the environment
        Bob::ThrEnvProg::stop();
        // Terminate Bob++
        Bob::core::End();
        // Delete the allocated instance
        delete Instance;
}

 * \endcode
 *
 */


/** Class to represent a thread mutex
 */
struct ThrMutex {
  /// The POSIX mutex
  pthread_mutex_t mut;
public:
  /// Constructor
  ThrMutex() {
    pthread_mutex_init(&mut, 0);
  };
  /// destructor
  virtual ~ThrMutex() {
    pthread_mutex_destroy(&mut);
  };
  /// Method to lock the mutex
  virtual void Lock() {
    pthread_mutex_lock(&mut);
  }
  /// Method to unlock the mutex
  virtual void Unlock() {
    pthread_mutex_unlock(&mut);
  }
};

/** Class to implement a barrier between the Threads
 *
 */
struct ThrBarrier {
  /// The POSIX mutex
  pthread_mutex_t mut;
  /// The POSIX condition variable
  pthread_cond_t cond;
  /// the number of thread for the barrier
  int n;
  /// the number of thread that currently wait for the barrier
  volatile int nbw;
public:
  /// Constructor
  ThrBarrier(int _n = 0): n(_n), nbw(0) {
    pthread_cond_init(&cond, 0);
    pthread_mutex_init(&mut, 0);
  };
  /// Destructor
  virtual ~ThrBarrier() {
    pthread_cond_destroy(&cond);
    pthread_mutex_destroy(&mut);
  };
  /// Set the number of Thread to wait.
  virtual void setNb(int _n) {
    n = _n;
  }
  /// Method called by the different thread to wait, the last one will broadcast to everyone.
  virtual void Wait() {
    pthread_mutex_lock(&mut);
    nbw++;
    if (n <= nbw) {
      pthread_cond_broadcast(&cond);
      pthread_mutex_unlock(&mut);
    } else {
      pthread_cond_wait(&cond, &mut);
      pthread_mutex_unlock(&mut);
    }
  }
};

/** Class to represent a thread
 */
struct Thread {
  /// Id of the thread
  pthread_t id;
  /// The condition of the thread (use to wait work)
  pthread_cond_t cond;
  /// the mutex associated with the condition variable
  pthread_mutex_t mut;
  /// pointer on a Algorithm
  base_Algo *a;
  /// the barrier to synchronize the thread after an execution on an algo.
  ThrBarrier *bar;
  /// the attribut to modify the stack size
  pthread_attr_t attr;
  /// enumeration for the state of a thread
  enum StateThr {ThWait, /*!< the thread wait for works */
                 ThHasWork, /*!< the thread has work but have not begin to process it. */
                 ThWork, /*!< the thread is working */
                 ThStop/*!< the thread is stopped */
                };
  /// the State of a thread
  StateThr state;
public:
  /// Constructor
  Thread();
  /// Destructor
  virtual ~Thread() { }
  /// The main loop
  void loop();
  /// Method to run an algorithm, the algorithm must be ready
  void run(base_Algo *al, ThrBarrier *thb) {
    pthread_mutex_lock(&mut);
    if (state != ThWait) {
      pthread_mutex_unlock(&mut);
    }
    a = al;
    bar = thb;
    state = ThHasWork;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&mut);
  }
  /// Method to stop the thread
  void stop() {
    pthread_mutex_lock(&mut);
    state = ThStop;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&mut);
  }
  /// Methods to get the pthread_id of a the thread.
  pthread_t thid() {
    return id;
  }
};

/** Class to have the Bob identifier of a thread...
 */
class ThrId: public EnvId {
  strbuff<10> num;
public:
  /// Constructor
  ThrId(): EnvId(), num() {}
  /// Destructor
  virtual ~ThrId() { }
  /// the id() method
  virtual long id();
  /// the id() method
  virtual long nb();
};

/** The solution data structure for the Threaded environment
 */
template<class Trait>
class ThrGoal: public Trait::Goal {
public :
  /// Type of a Node
  typedef typename Trait::Node TheNode;
  /// Type of the Instance
  typedef typename Trait::Instance TheInstance;
  /// Type for the goal of the search
  typedef typename Trait::Goal TheGoal;
  /// Type of the Statistics
  typedef typename Trait::Stat TheStat;
protected:
  /// the mutex used to insure access to the real data in mutual exclusion mode.
  pthread_mutex_t mut;
public:
  /// Constructor
  ThrGoal(bool b) : TheGoal(b) {
    pthread_mutex_init(&mut, 0);
  }
  /// Destructor
  virtual ~ThrGoal() {
    pthread_mutex_destroy(&mut);
  }
  ///Init method
  /** Intialiaze the Goal.
     * @param i Instance used to initialize the Goal
     */
  void Init(TheInstance *i) {
    pthread_mutex_lock(&mut);
    TheGoal::Init(i);
    pthread_mutex_unlock(&mut);
  }
  /** Method to test if the node is suitable to be explore later.
    * @param n the node to test
    */
  virtual bool is4Search(TheNode *n) {
    bool b;
    pthread_mutex_lock(&mut);
    b = TheGoal::is4Search(n);
    pthread_mutex_unlock(&mut);
    return b;
  }
  /** Method called when a node is a solution
    * @param n the solution node
    */
  virtual bool updSol(TheNode *n) {
    bool b;
    pthread_mutex_lock(&mut);
    b = TheGoal::updSol(n);
    pthread_mutex_unlock(&mut);
    return b;
  }
  /** The merging method is used to update the goal with another one.
    * this method is not really used in this environmment.
    * @param g TheGoal used to updated the Goal.
    */
  virtual bool current_merge(const TheGoal *g) {
    bool b;
    pthread_mutex_lock(&mut);
    b = TheGoal::current_merge(g);
    pthread_mutex_unlock(&mut);
    return b;
  }
  /** The merging method is used to update the goal with another one.
    * this method is not really used in this environmment.
    * @param g TheGoal used to updated the Goal.
    */
  virtual bool final_merge(const TheGoal *g) {
    bool b;
    pthread_mutex_lock(&mut);
    b = TheGoal::final_merge(g);
    pthread_mutex_unlock(&mut);
    return b;
  }

};

/** Class to represent the priority queue use by several threads
 *
 * This class uses the Bob::ThrMutex for the access to the
 * priority queue that stores the pending nodes.
 *
 * This class is reponsable of the terminaison of the algorithm
 * if the priority queue is empty, a thread have to wait, because
 * another thread could insert a new node.
 * The terminaison arrives if all the threads wait that new node
 * will be inserted.
 */
template<class Node, class PriComp>
class ThrPQ : public PQ<Node, PriComp> {
  pthread_cond_t cond; // the condition variable
  pthread_mutex_t mut; // the associated mutex.
  int Wth;             // number of waiting threads
  PQ<Node, PriComp> *pq;         // the local priority queue.
public:

  /// Constructor
  ThrPQ(PQ<Node, PriComp> *pqp) : PQ<Node, PriComp>() {
    pq = pqp;
    pthread_cond_init(&cond, 0);
    pthread_mutex_init(&mut, 0);
    Wth = 0;
  }
  /// Destructor
  virtual ~ThrPQ() {
    pthread_cond_destroy(&cond);
    pthread_mutex_destroy(&mut);
  }
  ///Reset method
  void Reset() {
    pq->Reset();
  }
  /// Remove a reference on the Priority Queue:w
  virtual void remRef() {
    pthread_mutex_lock(&mut);
    PQ<Node, PriComp>::remRef();
    pthread_mutex_unlock(&mut);
  }
  /// Add a reference on the Priority Queue:w
  virtual void addRef() {
    pthread_mutex_lock(&mut);
    PQ<Node, PriComp>::addRef();
    pthread_mutex_unlock(&mut);
  }
  /// Insertion
  void Ins(Node *n) {
    n->addRef();
    pthread_mutex_lock(&mut);
    pq->Ins(n);
    if (Wth > 0) {
      pthread_cond_signal(&cond);
      pthread_mutex_unlock(&mut);
    } else pthread_mutex_unlock(&mut);
  }
  virtual int DelG() {
    return 0;
  }
  /// Delete the best node
  Node *Del() {
    Node *n;
    pthread_mutex_lock(&mut);
    n = pq->Del();
    pthread_mutex_unlock(&mut);
    if (n != 0) {
      n->remRef();
    } else {
      pthread_mutex_lock(&mut);
      Wth++;
      if (Wth >= PQ<Node, PriComp>::ref) {
        Wth = 100000;
        pthread_cond_broadcast(&cond);
        pthread_mutex_unlock(&mut);
        return 0;
      }
      pthread_cond_wait(&cond, &mut);
      pthread_mutex_unlock(&mut);
      pthread_mutex_lock(&mut);
      while ((n = pq->Del()) == 0) {
        if (Wth >= PQ<Node, PriComp>::ref) {
          Wth = 100000;
          pthread_cond_broadcast(&cond);
          pthread_mutex_unlock(&mut);;
          return 0;
        }
        pthread_cond_wait(&cond, &mut);
        pthread_mutex_unlock(&mut);
        pthread_mutex_lock(&mut);
      }
      Wth--;
      pthread_mutex_unlock(&mut);
    }
    return n;
  }
  /// Prints the statistics and contents
  ostream &Prt(ostream &os = std::cout) const {
    pq->Prt(os);
    return os;
  }
  /// Return the statistics
  PQStat *getStat() {
    return pq->getStat();
  }
};

/** The threaded Environment static class
 */
class ThrEnvProg {
protected:
  /// Array of threads
  static Thread *tt;
  /// Number of threads
  static int nbth;
  /// Boolean to know if the instance is duplicated in each thread/algo or not.
  static bool multi_inst;
public:
  /// Constructor
  ThrEnvProg() { }
  /// the init method of the environment
  static void init() {
    std::cout << "Init du ThrEnvProg\n";
    core::opt().add(std::string("--thr"), Property("-n", "Number of thread used", 2, &nbth));
    core::opt().add(std::string("--thr"), Property("-t", "used one instance per algorithm", &multi_inst));
    core::opt().add(std::string("--thr"), Property("-s", "the Stack size used by each thread", (int)(PTHREAD_STACK_MIN*10)));
    core::log().setEId(new ThrId());
  }
  /// The start method of the environment, the real threads are created
  static void start() {
    std::cout << "In start:" << nbth << "\n";
    if (nbth == 0) {
      std::cerr << "Number of thread 0\n";
      exit(1);
    }
    tt = new Thread[nbth];
  }
  /// The stop method of the environment, the real threads are stopped
  static void stop() {
    for (int i = 0 ; i < nbth; i++) {
      tt[i].stop();
      pthread_join(tt[i].id, NULL);
    }
    delete[] tt;
  }
  /// Search the rank of a thread.
  static int rank() {
    int i;
    if (tt == 0) return 0;
    for (i = 0; i < nthread(); i++) {
      if (pthread_equal(tt[i].thid(), pthread_self())) {
        return i;
      }
    }
    return 0;
  }
  /// get the value of the option multi_inst.
  static bool instance_multi() {
    return multi_inst;
  }
  /// get the default stack size
  static size_t stack_size() {
    return (size_t)core::opt().NVal("--thr", "-s");
  }
  /// Method that returns the number of active threads
  static int nthread() {
    return nbth;
  }
  /** Method to give an Algo to solve to a thread
   * @param i the indice of the thread
   * @param a an algo to compute.
   * @param thb the barrier to synchronize all the thread at the end of the execution of the algorithm.
   */
  static void run(int i, base_Algo *a, ThrBarrier *thb) {
    tt[i].run(a, thb);
  }
};


/** The Threaded Wrapper for Algorithms.
 */
template<class Trait>
class ThrAlgoEnvProg : public AlgoEnvProg<Trait> {
  typedef typename Trait::Instance TheInstance;
  typedef typename Trait::Node TheNode;
  typedef typename Trait::Algo TheAlgo;
  typedef typename Trait::Goal TheGoal;
  typedef typename Trait::Stat TheStat;
  typedef typename Trait::PriComp ThePriComp;
  typedef AlgoEnvProg<Trait> aep;

private:
  TheAlgo **pal;
  TheInstance **thinst;
  TheStat **st;
  PQSkew<TheNode, ThePriComp> *pqs;
  ThrPQ<TheNode, ThePriComp> *thpq;
  ThrGoal<Trait> *thrgoal;
protected:
  /** Method to Allocate all the object used by the environment.
    * Many of them are given by the Tait Class.
    */
  virtual void Alloc(TheInstance *inst) {
    pqs = new PQSkew<TheNode, ThePriComp>(true);
    BOBASSERT(pqs != 0, Exception, "Threaded Env", "PQSkew allocation failed");
    thpq = new ThrPQ<TheNode, ThePriComp>(pqs);
    BOBASSERT(thpq != 0, Exception, "Threaded Env", "ThrPq allocation failed");
    thrgoal = new ThrGoal<Trait>(true);
    BOBASSERT(thrgoal != 0, Exception, "Threaded Env", "ThrGoal allocation failed");
    st = new TheStat *[ThrEnvProg::nthread()];
    BOBASSERT(st != 0, Exception, "Threaded Env", "Tab Stat allocation failed");
    pal = new TheAlgo *[ThrEnvProg::nthread()];
    BOBASSERT(pal != 0, Exception, "Threaded Env", "Tab Algo allocation failed");
    if (ThrEnvProg::instance_multi()) {
      thinst = new TheInstance *[ThrEnvProg::nthread()];
      BOBASSERT(theinst != 0, Exception, "Threaded Env", "Tab Instance allocation failed");
    }  else thinst = 0;
    for (int i = 0 ; i < ThrEnvProg::nthread() ; i++) {
      if (ThrEnvProg::instance_multi()) {
        thinst[i] = new TheInstance(*inst);
        BOBASSERT(thinst[i] != 0, Exception, "Threaded Env", "Instance allocation failed");
        inst = thinst[i];
      }
      st[i] = new TheStat(inst->id(), thpq->getStat(), thrgoal->getStat());
      BOBASSERT(st[i] != 0, Exception, "Threaded Env", "Stat allocation failed");
      pal[i] = new TheAlgo(inst, thpq, thrgoal, st[i]);
      BOBASSERT(pal[i] != 0, Exception, "Threaded Env", "Algo allocation failed");
    }
    st[0]->namelog();
  }
  /** Method to Display the Statistics and the Goal
    */
  virtual void display_stat() {
    pal[0]->getStat()->PrtL(std::cout);
    for (int i = 0 ; i < ThrEnvProg::nthread() ; i++) {
      pal[i]->getStat()->PrtD(std::cout);
    }
    pqs->getStat()->PrtL(std::cout);
    pqs->getStat()->PrtD(std::cout);
    thrgoal->getStat()->PrtL(std::cout);
    thrgoal->getStat()->PrtD(std::cout);
  }
  /** Method to Display the Statistics and the Goal
    */
  virtual void display_goal() {
    thrgoal->Prt(std::cout);
  }
  /** Method to Free all the object allocated in the Alloc Method.
    */
  virtual void Free() {
    for (int i = 0 ; i < ThrEnvProg::nthread() ; i++) {
      delete pal[i];
      if (ThrEnvProg::instance_multi()) {
         delete thinst[i];
      }
    }
    if (ThrEnvProg::instance_multi()) {
        delete[] thinst;
    }
    delete[] pal;
    delete thpq;
    delete pqs;
  }

  /** doit is the method that do the job for the two interface.
   * @param nd the node to explore from.
   * @param inst the instance associated with the search.
   */
  void doit(TheNode *nd, TheInstance *inst) {
    ThrBarrier thb(ThrEnvProg::nthread());
    inst->Init();
    Alloc(inst);
    thrgoal->Init(inst);
    if (nd==0 ) inst->InitAlgo(pal[0]);
    else pal[0]->GPQIns(nd);
    for (int i = 1 ; i < ThrEnvProg::nthread() ; i++) {
      ThrEnvProg::run(i, pal[i], &thb);
    }
    (*(pal[0]))();
    aep::display();
    thb.Wait();
    Free();
  }
public:
  /// the constructor
  ThrAlgoEnvProg() : AlgoEnvProg<Trait>() {
  }
  /// the constructor
  virtual ~ThrAlgoEnvProg() {
    for (int i = 0 ; i < ThrEnvProg::nthread() ; i++) {
      delete st[i];
    }
    delete thrgoal;
    delete[] st;
  }
  /** get the goal after a resolution
   * @return the goal with the solution.
   */
  TheGoal *goal() {
    return thrgoal;
  }
  /** get the stat after a resolution
   * @return the stat.
   */
  TheStat *stat() {
    return st[0];
  }
  /** solve the problem
   * @param inst the instance to solve
   */
  virtual void operator()(TheInstance *inst) {
      doit(0,inst);
  }
  /** solve the problem from the node \a nd
   * @param inst the instance to solve
   * @param nd the root node to solve
   */
  virtual void operator()(TheNode *nd, TheInstance *inst) {
      doit(nd,inst);
  }
};


/** The Threaded Algorithms Wrapper for Branch and Bound.
 */
template<class Trait>
class ThrBBAlgoEnvProg : public ThrAlgoEnvProg<Trait> {
public:
  /// the constructor
  ThrBBAlgoEnvProg() : ThrAlgoEnvProg<Trait>() {
  }
  /// Destructor
  virtual ~ThrBBAlgoEnvProg() {}
};

/** The Threaded Algorithms Wrapper for simple Divide and Conquer.
 */
template<class Trait>
class ThrSDCAlgoEnvProg : public ThrAlgoEnvProg<Trait> {
public:
  /// the constructor
  ThrSDCAlgoEnvProg() : ThrAlgoEnvProg<Trait>() {
  }
  /// Destructor
  virtual ~ThrSDCAlgoEnvProg() {}
};

/** The Threaded Algorithms Wrapper for Costed Divide and Conquer.
 */
template<class Trait>
class ThrCDCAlgoEnvProg : public ThrAlgoEnvProg<Trait> {
public:
  /// the constructor
  ThrCDCAlgoEnvProg() : ThrAlgoEnvProg<Trait>() {
  }
  /// Destructor
  virtual ~ThrCDCAlgoEnvProg() {}
};

/**
 * @}
 */

} // End of the namspace
#endif

